"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _yesCore = require("yes-core");

require("./exprhandlermap");

require("./baseexprhandler");

var _formdepend = require("./formdepend");

var DefaultExprHandler = _yesCore.YIUI.extend(_yesCore.YIUI.BaseExprHandler, {
  doCalcItemInit: function doCalcItemInit(meta) {
    var metaKey = meta.key;
    var items = [];

    if (metaKey) {
      var binding = meta.dataBinding;
      var defaultValue = binding ? binding.defaultValue : '';
      var formulaValue = binding ? binding.defaultFormulaValue : '';
      var valueDependency = binding ? binding.valueDependency : '';
      var item = this.createCalcItem(metaKey, metaKey, _formdepend.CalcItem.Head, defaultValue, formulaValue, valueDependency);
      items.push(item);
    }

    return items;
  },
  doEnableItemInit: function doEnableItemInit(meta, tree) {
    var metaKey = meta.key;

    if (metaKey) {
      var item = this.createEnableItem(metaKey, metaKey, _formdepend.EnableItem.Head, meta.enable, meta.enableDependency);
      tree.add(item);
    }
  },
  doVisibleItemInit: function doVisibleItemInit(meta, tree) {
    var metaKey = meta.key;

    if (metaKey) {
      var item = this.createVisibleItem(metaKey, metaKey, _formdepend.VisibleItem.Head, meta.visible, meta.visibleDependency);
      tree.add(item);
    }
  },
  doCheckItemInit: function doCheckItemInit(meta, tree) {
    var metaKey = meta.key;

    if (metaKey) {
      var binding = meta.dataBinding;
      var checkRule = binding ? binding.checkRule : '';
      var errorInfo = binding ? binding.errorInfo : '';
      var isRequired = binding ? binding.required : '';
      var checkDependency = binding ? binding.checkDependency : '';
      var item = this.createCheckItem(metaKey, metaKey, _formdepend.CheckItem.Head, checkRule, errorInfo, isRequired, checkDependency);
      tree.add(item);
    }
  },
  doOperationEnableInit: function doOperationEnableInit(tree) {
    var self = this;
    this.form.metaForm.operations.forEach(function (opt) {
      var item = self.createEnableItem(opt.key, opt.key, _formdepend.EnableItem.Operation, opt.enableCnt, opt.enableDependency);
      tree.add(item);
    });
  },
  doOperationVisibleInit: function doOperationVisibleInit(tree) {
    var self = this;
    this.form.metaForm.operations.forEach(function (opt) {
      var item = self.createVisibleItem(opt.key, opt.key, _formdepend.VisibleItem.Operation, opt.visibleCnt, opt.visibleDependency);
      tree.add(item);
    });
  },
  doGlobalCheckInit: function doGlobalCheckInit(tree) {
    var metaForm = this.form.metaForm;
    var items = this.createCheckItems(metaForm.key, metaForm.key, metaForm.uichecks);
    tree.setGlobalItems(items);
  }
});

_yesCore.YIUI.ExprHandlerMap.getInstance().reg(0, DefaultExprHandler);

var _default = DefaultExprHandler;
exports["default"] = _default;