"use strict";

var _yesCore = require("yes-core");

require("./exprhandlermap");

require("./baseexprhandler");

var _formdepend = require("./formdepend");

var ListViewExprHandler = _yesCore.YIUI.extend(_yesCore.YIUI.BaseExprHandler, {
  doCalcItemInit: function doCalcItemInit(metaLV) {
    var columns = metaLV.columnInfo;
    var self = this,
        items = [];
    var index = 0;
    columns.forEach(function (column) {
      var colKey = column.key;

      if (colKey) {
        var binding = column.dataBinding;
        var defaultValue = binding ? binding.defaultValue : '';

        if (!defaultValue) {
          defaultValue = column.defaultValue;
        }

        var formulaValue = binding ? binding.defaultFormulaValue : '';

        if (!formulaValue) {
          formulaValue = column.formulaValue;
        }

        var valueDependency = binding ? binding.valueDependency : '';

        if (!valueDependency) {
          valueDependency = column.valueDependency;
        }

        var item = self.createCalcItem(colKey, metaLV.key, _formdepend.CalcItem.List, defaultValue, formulaValue, valueDependency);
        var pos = new _formdepend.ExprItemPos();
        pos.setIndex(index);
        item.setPos(pos);
        items.push(item);
      }

      index++;
    });
    return items;
  },
  doEnableItemInit: function doEnableItemInit(metaLV, tree) {
    var metaKey = metaLV.key;
    var self = this;

    if (metaKey) {
      var item = self.createEnableItem(metaKey, metaKey, _formdepend.EnableItem.Head, metaLV.enable, metaLV.enableDependency);
      tree.add(item);
      var itemSet = new _formdepend.EnableItemSet(metaKey);
      tree.add(itemSet);
      var index = 0;
      metaLV.columnInfo.forEach(function (column) {
        var colKey = column.key;

        if (colKey) {
          var item = self.createEnableItem(colKey, metaKey, _formdepend.EnableItem.List, column.enable, column.enableDependency);
          var pos = new _formdepend.ExprItemPos();
          pos.setIndex(index);
          item.setPos(pos);
          itemSet.add(item);
        }

        index++;
      });
    }
  },
  doVisibleItemInit: function doVisibleItemInit(metaLV, tree) {
    var metaKey = metaLV.key;
    var self = this;

    if (metaKey) {
      var item = self.createVisibleItem(metaKey, metaKey, _formdepend.VisibleItem.Head, metaLV.visible, metaLV.visibleDependency);
      tree.add(item);
      var itemSet = new _formdepend.VisibleItemSet(metaKey);
      tree.add(itemSet);
      var index = 0;
      metaLV.columnInfo.forEach(function (column) {
        var colKey = column.key;

        if (colKey) {
          var item = self.createVisibleItem(colKey, metaKey, _formdepend.VisibleItem.Column, column.visible, column.visibleDependency);
          var pos = new _formdepend.ExprItemPos();
          pos.setIndex(index);
          item.setPos(pos);
          itemSet.add(item);
        }

        index++;
      });
    }
  },
  doCheckItemInit: function doCheckItemInit(metaLV, tree) {
    var metaKey = metaLV.key;
    var self = this;

    if (metaKey) {
      var itemSet = new _formdepend.VisibleItemSet(metaKey);
      tree.add(itemSet);
      var index = 0;
      metaLV.columnInfo.forEach(function (column) {
        var colKey = column.key;

        if (colKey) {
          var binding = column.dataBinding;
          var checkRule = binding ? binding.checkRule : '';
          var errorInfo = binding ? binding.errorInfo : '';
          var isRequired = binding ? binding.required : '';
          var checkDependency = binding ? binding.checkDependency : '';
          var item = self.createCheckItem(colKey, metaKey, _formdepend.CheckItem.List, checkRule, errorInfo, isRequired, checkDependency);
          var pos = new _formdepend.ExprItemPos();
          pos.setIndex(index);
          item.setPos(pos);
          itemSet.add(item);
        }

        index++;
      });
    }
  }
});

_yesCore.YIUI.ExprHandlerMap.getInstance().reg(_yesCore.YIUI.CONTROLTYPE.TILEDLIST, ListViewExprHandler);

_yesCore.YIUI.ExprHandlerMap.getInstance().reg(_yesCore.YIUI.CONTROLTYPE.GALLERY, ListViewExprHandler);

_yesCore.YIUI.ExprHandlerMap.getInstance().reg(_yesCore.YIUI.CONTROLTYPE.ROTATOR, ListViewExprHandler);

_yesCore.YIUI.ExprHandlerMap.getInstance().reg(_yesCore.YIUI.CONTROLTYPE.ROTATORLIST, ListViewExprHandler);

_yesCore.YIUI.ExprHandlerMap.getInstance().reg(_yesCore.YIUI.CONTROLTYPE.WATERFALL, ListViewExprHandler);

_yesCore.YIUI.ExprHandlerMap.getInstance().reg(_yesCore.YIUI.CONTROLTYPE.LISTVIEW, ListViewExprHandler);

_yesCore.YIUI.ExprHandlerMap.getInstance().reg(_yesCore.YIUI.CONTROLTYPE.LISTLAYOUTVIEW, ListViewExprHandler);